import discord
from cpl_core.logging import LoggerABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List
from discord import RawReactionActionEvent

from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.auto_role import AutoRole
from bot_data.model.auto_role_rule import AutoRoleRule


class ReactionHandler:

    def __init__(
            self,
            logger: LoggerABC,
            bot: DiscordBotServiceABC,
            servers: ServerRepositoryABC,
            auto_roles: AutoRoleRepositoryABC
    ):
        self._logger = logger
        self._bot = bot
        self._servers = servers
        self._auto_roles = auto_roles

        self._message_ids = self._auto_roles.get_auto_roles().select(lambda x: x.discord_message_id)
        self._roles = self._auto_roles.get_auto_roles()

    async def handle(self, payload: RawReactionActionEvent, r_type=None) -> None:
        self._logger.trace(__name__, f'Handle reaction {payload} {r_type}')
        if payload.message_id not in self._message_ids:
            self._logger.debug(__name__, f'Message not in auto-roles - skipping')
            return

        guild = self._bot.get_guild(payload.guild_id)
        user = guild.get_member(payload.user_id)
        if user is None:
            self._logger.warn(__name__, f'User {payload.user_id} in {guild.name} not found - skipping')
            return

        if user.bot:
            self._logger.debug(__name__, f'User is bot - skipping')
            return

        emoji = payload.emoji.name
        auto_role: AutoRole = self._roles.where(lambda x: x.discord_message_id == payload.message_id).single_or_default()
        if auto_role is None:
            self._logger.debug(__name__, f'auto-role for message not found - skipping')
            return

        rules: List[AutoRoleRule] = self._auto_roles.get_auto_role_rules_by_auto_role_id(auto_role.auto_role_id)
        if rules.count() == 0:
            self._logger.debug(__name__, f'auto-role rules not found - skipping')
            return

        for rule in rules:
            if emoji != rule.emoji_name:
                continue

            if r_type == 'add':
                role = guild.get_role(rule.role_id)
                self._logger.debug(__name__, f'Assign role {role.name} to {user.name}')
                await user.add_roles(role)
            elif r_type == 'remove':
                role = guild.get_role(rule.role_id)
                self._logger.debug(__name__, f'Remove role {role.name} to {user.name}')
                await user.remove_roles(role)
            else:
                self._logger.warn(__name__, f'Invalid reaction type {r_type}')
