import asyncio

from cpl_core.configuration import ConfigurationABC
from cpl_discord.command import DiscordCommandABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.configuration.server_settings import ServerSettings
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class PurgeCommand(DiscordCommandABC):

    def __init__(
            self,
            logger: CommandLogger,
            config: ConfigurationABC,
            message_service: MessageServiceABC,
            permissions: PermissionServiceABC,
            client_utils: ClientUtilsServiceABC,
            translate: TranslatePipe
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._config = config
        self._message_service = message_service
        self._permissions = permissions
        self._client_utils = client_utils
        self._t = translate

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.hybrid_command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def purge(self, ctx: Context):
        self._logger.debug(__name__, f'Received command purge {ctx}')
        server_settings: ServerSettings = self._config.get_configuration(f'ServerSettings_{ctx.guild.id}')

        await self._message_service.send_ctx_msg(ctx, self._t.transform('modules.moderator.purge_message'))
        await asyncio.sleep(server_settings.message_delete_timer)
        try:
            await ctx.channel.purge()
        except Exception as e:
            self._logger.error(__name__, f'Cannot purge channel {ctx.channel.id}', e)
            await self._message_service.send_ctx_msg(ctx, self._t.transform('common.bot_has_no_permission_message'))

        self._logger.trace(__name__, f'Finished purge command')
