import asyncio

from cpl_core.configuration import ConfigurationABC
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.configuration.bot_settings import BotSettings
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class RestartCommand(DiscordCommandABC):

    def __init__(
            self,
            logger: CommandLogger,
            config: ConfigurationABC,
            message_service: MessageServiceABC,
            bot: DiscordBotServiceABC,
            client_utils: ClientUtilsServiceABC,
            translate: TranslatePipe,
            permissions: PermissionServiceABC,
            settings: BotSettings
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._config = config
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils
        self._t = translate
        self._permissions = permissions
        self._settings = settings

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.hybrid_command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def restart(self, ctx: Context):
        self._logger.debug(__name__, f'Received command restart {ctx}')

        self._config.add_configuration('IS_RESTART', 'true')
        await self._client_utils.presence_game('common.presence.restart')
        await self._message_service.send_ctx_msg(ctx, self._t.transform('modules.admin.restart_message'))
        await asyncio.sleep(self._settings.wait_for_restart)
        await self._bot.stop_async()

        self._logger.trace(__name__, f'Finished restart command')
